/*
 * #© Copyright 2019 - Micro Focus or one of its affiliates
 * #
 * # The only warranties for products and services of Micro Focus and its affiliates and licensors (“Micro Focus”)
 * # are as may be set forth in the express warranty statements accompanying such products and services.
 * # Nothing herein should be construed as constituting an additional warranty.
 * # Micro Focus shall not be liable for technical or editorial errors or omissions contained herein.
 * # The information contained herein is subject to change without notice.
 *
 */

'use strict';

var path = require('path'),
  util = require('util'),
  misc = require('../lib/misc.js'),
  logger = require('../lib/logger.js');

(function (argv) {
  var loggerOpts, testDir, engine, cloudOpts, testBeginTM;

  /* test settings */
  testBeginTM = Date.now();
  testDir = argv.testdir || process.cwd();
  testDir = path.resolve(testDir);

  /* startup logger */
  function getDefaultLogFilename() {
    var filename;
    filename = util.format('truapi_%d_%d.log', process.pid, testBeginTM);
    return path.join(testDir, filename);
  }
  loggerOpts = {
    logLevel: argv.logLevel || 'debug',
    silent: argv.silent ? true : false,
    logFile: argv.logFile || getDefaultLogFilename()
  };
  logger.setOptions(loggerOpts);
  logger.debug('TruAPI CLI started');

  /* startup engine 
   * TODO check the storm_instance_id
   */
  cloudOpts = {
    snd: argv.storm_send || 'tcp://127.0.0.1:5564',
    rcv: argv.storm_recv || 'tcp://127.0.0.1:5563',
    testId: argv.storm_test_id || 'cloudTest',
    groupId: argv.storm_group_id || 'cloudTestGroup',
    instanceId: argv.storm_instance_id
  };
  engine = new require('../lib/engine_cloud.js').EngineCloud(cloudOpts);

  /* load script from folder */
  if (!engine.loadFromFolder(testDir)) {
    logger.error('Cannot load test. testDir = %s', testDir);
    process.exit(misc.procExitCode.badTest);
  }

  /* run tests */
  engine.run(function (err) {
    var exitCode;
    exitCode = misc.procExitCode.ok;
    if (err) {
      logger.error('Test finished with error. Err = %s', err.toString());
      exitCode = misc.procExitCode.badTest;
    }
    else {
      logger.info('Test finished');
    }
    process.exit(exitCode);
  });
})(require('optimist').argv);

