#!/usr/bin/env node
/*
 * #© Copyright 2019 - Micro Focus or one of its affiliates
 * #
 * # The only warranties for products and services of Micro Focus and its affiliates and licensors (“Micro Focus”)
 * # are as may be set forth in the express warranty statements accompanying such products and services.
 * # Nothing herein should be construed as constituting an additional warranty.
 * # Micro Focus shall not be liable for technical or editorial errors or omissions contained herein.
 * # The information contained herein is subject to change without notice.
 *
 */

'use strict';

var path = require('path'),
  _ = require('underscore'),
  generator = require('../lib/test_generator.js'),
  misc = require('../lib/misc.js'),
  procExitCode = misc.procExitCode,
  logger = require('../lib/logger.js'),
  optimist = require('optimist')
    .alias('h', 'help')
    .describe('testdir', 'The test script folder')
    .describe('testUrl', 'The target URL')
    .describe('testname', 'The test name')
    .describe('template', 'The template name')
    .usage('TruAPI test project generator');

(function (argv) {
  var loggerOpts, testDir, testGen, genOpts, testOpts, pkg, pkgName;

  if (argv.help) {
    optimist.showHelp();
    process.exit(procExitCode.usage);
  }

  /* startup logger */
  loggerOpts = {
    logLevel: argv.logLevel || 'debug',
    silent: argv.silent ? true : false
  };
  logger.setOptions(loggerOpts);
  logger.debug('TruAPI CLI started');

  /* check arguments */
  testDir = argv.testdir || argv.testDir || process.cwd();
  testDir = path.resolve(testDir);

  try {
    pkg = require(path.join(testDir, './package.json'));
    if (pkg && pkg.name) {
      if (_.isString(pkg.name) && pkg.name.trim().length > 0) {
        pkgName = pkg.name.trim();
      }
    }
  } catch(ex) {
  }

  testOpts = {
    dir: testDir,
    testName: argv.testname || pkgName || path.basename(testDir),
    testUrl: argv.testUrl || argv.testurl || ''
  };

  /* create generator */
  genOpts = {
    templatesDir: path.join(__dirname, '../templates'),
    template: argv.template || 'default'
  };
  testGen = generator.create(genOpts);
  if (!testGen) {
    logger.error('Cannot load template');
    process.exit(procExitCode.badTemplate);
    return;
  }

  testGen.gen(testOpts, function (err) {
    if (err) {
      logger.error('test generator error %s', err.toString());
      return;
    }
    logger.info('TruAPI test created in %s', testDir);
  });
})(optimist.argv);

