/*
 * #© Copyright 2019 - Micro Focus or one of its affiliates
 * #
 * # The only warranties for products and services of Micro Focus and its affiliates and licensors (“Micro Focus”)
 * # are as may be set forth in the express warranty statements accompanying such products and services.
 * # Nothing herein should be construed as constituting an additional warranty.
 * # Micro Focus shall not be liable for technical or editorial errors or omissions contained herein.
 * # The information contained herein is subject to change without notice.
 *
 */

'use strict';

var util = require('util'),
  events = require('events'),
  logger = require('./logger.js'),
  misc = require('./misc.js'),
  dbgTrace = misc.dbgTrace('engCloudZMQ'),
  fieldsName = misc.fieldsName,
  zmq = require('zeromq');

function EngConnector(opts) {
  var self = this;

  if (!(this instanceof EngConnector)) {
    return new EngConnector(opts);
  }
  events.EventEmitter.call(this);

  this._opts = opts;
  this._cloudOpts = opts.cloudOpts || {};
  this._onMsg = opts.onMsg || function () {};

  /* Startup zmq subscribe & push
   * NOTE: must init the subscribe before we init push.
   * The receiver must be ready before the sender.
   */

  /* startup zmq subscribe */
  logger.debug('connect zmq sub [ %s ]', this._cloudOpts.rcv);
  this._subscribe = zmq.socket('sub');
  this._subscribe.setsockopt(zmq.ZMQ_RCVHWM, 20000);
  this._subscribe.connect(this._cloudOpts.rcv);
  logger.debug('reg zmq sub [ %s ]', this._cloudOpts.instanceId);
  this._subscribe.subscribe(this._cloudOpts.instanceId);
  this._subscribe.on('error', function (err) {
    if (err) {
      logger.error('zmq subscribe error %s', err.toString());
    }
  });
  this._subscribe.on('message', function (instId, data) {
    var msg, instIdStr;
    dbgTrace('zmq subscribe, new msg');
    instIdStr = instId.toString();
    if (instIdStr !== self._cloudOpts.instanceId) {
      logger.error('invalid instanceId');
      return;
    }
    try {
      msg = JSON.parse(data.toString());
    }
    catch (err) {
      logger.error('invalid msg data, err %s', err.toString());
      return;
    }
    self._onMsg(msg);
  });

  /* startup zmq push */
  logger.debug('create zmq push [ %s ].', this._cloudOpts.snd);
  this._push = zmq.socket('push');
  this._push.setsockopt(zmq.ZMQ_LINGER, 0);
  this._push.setsockopt(zmq.ZMQ_SNDHWM, 20000);
  this._push.connect(this._cloudOpts.snd);
  this._push.on('error', function (err) {
    if (err) {
      logger.error('zmq push error %s', err.toString());
    }
  });
}

util.inherits(EngConnector, events.EventEmitter);
exports.EngConnector = EngConnector;

EngConnector.prototype.send = function (name, data) {
  var sndData;
  dbgTrace('snd %s to zmq', name);
  sndData = {};
  sndData[fieldsName.evtName] = name;
  sndData[fieldsName.evtData] = data;
  try {
    this._push.send(JSON.stringify(sndData));
  }
  catch (err) {
    logger.error('zmq send error %s', err.toString());
  }
};

