/*
 * #© Copyright 2019 - Micro Focus or one of its affiliates
 * #
 * # The only warranties for products and services of Micro Focus and its affiliates and licensors (“Micro Focus”)
 * # are as may be set forth in the express warranty statements accompanying such products and services.
 * # Nothing herein should be construed as constituting an additional warranty.
 * # Micro Focus shall not be liable for technical or editorial errors or omissions contained herein.
 * # The information contained herein is subject to change without notice.
 *
 */

'use strict';

var util = require('util'),
  events = require('events'),
  Runner = require('./runner.js').Runner,
  logger = require('./logger.js');

function Engine() {
  var self = this;
  if (!(this instanceof Engine)) {
    return new Engine();
  }
  events.EventEmitter.call(this);
  this._runner = new Runner();
  this._runner.on('subscribe', function (data) {
    self.onSubscribe(data);
  });
  this._runner.on('actionEnd', function (data) {
    self.onActionEnd(data);
  });
  this._runner.on('report', function (data) {
    self.onReport(data);
  });
  this._runner.on('vuserTerminated', function (data) {
    self.onVUserTerminated(data);
  });
}

util.inherits(Engine, events.EventEmitter);
exports.Engine = Engine;

Engine.prototype.onSubscribe = function (data) {
  throw new Error('Not Implemented');
};

Engine.prototype.onActionEnd = function (data) {
  throw new Error('Not Implemented');
};

Engine.prototype.onReport = function (data) {
  throw new Error('Not Implemented');
};

Engine.prototype.onVUserTerminated = function (data) {
  /* NOP */
};

Engine.prototype.load = function (opts) {
  var ret;
  opts = opts || {};
  logger.debug('load script %j', opts, {});
  ret = this._runner.load({
    script: opts.script,
    testId: opts.testId,
    groupId: opts.groupId
  });
  if (!ret) {
    logger.error('Cannot load script %s', opts.script);
  }
  return ret;
};

Engine.prototype.initVUsers = function (vusers, opts) {
  vusers = vusers || [];
  opts = opts || {};
  return this._runner.initVUsers(vusers, opts);
};

