/*
 * #© Copyright 2019 - Micro Focus or one of its affiliates
 * #
 * # The only warranties for products and services of Micro Focus and its affiliates and licensors (“Micro Focus”)
 * # are as may be set forth in the express warranty statements accompanying such products and services.
 * # Nothing herein should be construed as constituting an additional warranty.
 * # Micro Focus shall not be liable for technical or editorial errors or omissions contained herein.
 * # The information contained herein is subject to change without notice.
 *
 */

'use strict';

var yaCSV = require('ya-csv'),
  util = require('util'),
  fs = require('fs'),
  path = require('path'),
  _ = require('underscore'),
  misc = require('./misc.js'),
  logger = require('./logger.js'),
  dbgTrace = misc.dbgTrace('report');

exports.create = function () {
  var rstFiles;

  function init(opts) {
    var outDir, ts;
    dbgTrace('create report %j', opts);
    opts = opts || {};
    outDir = opts.outDir || process.cwd();

    /* init result  files */
    ts = opts.testBeginTM || Date.now();
    rstFiles = {
      message: {
        name: util.format('result_%d_message.csv', ts),
        hasWroteHead: false
      },
      transaction: {
        name: util.format('result_%d_transaction.csv', ts),
        hasWroteHead: false
      },
      dataPoint: {
        name: util.format('result_%d_datapoint.csv', ts),
        hasWroteHead: false
      }
    };
    _.each(_.keys(rstFiles), function (item) {
      var rstItem;
      rstItem = rstFiles[item];
      dbgTrace('init report %s, file %s', item, rstItem.name);
      try {
        rstItem.fd = fs.createWriteStream(path.join(outDir, rstItem.name));
        rstItem.csv = yaCSV.createCsvStreamWriter(rstItem.fd);
      }
      catch (err) {
        logger.error('Cannot create result file %s', rstItem.name);
      }
    });
    return rstFiles;
  }

  function getRcdIdHdr() {
    return ['groupId', 'testId', 'vuserId'];
  }

  function getRcdIdCsv(rcdId) {
    rcdId = rcdId || {};
    return [ rcdId.g || '', rcdId.t || '', rcdId.v || ''];
  }

  function addRcd(rcd) {
    var rstItem, hdr, csvLine, datKeys;
    rcd = rcd || {};
    dbgTrace('new record type %s', rcd.type);
    rstItem = rstFiles[rcd.type];
    if (!rstItem) {
      logger.warn('Unknown record type %s', rcd.type);
      return;
    }
    if (!rstItem.csv) {
      dbgTrace('Cannot write log file for %s', rcd.type);
      return;
    }
    datKeys = _.filter(_.keys(rcd.data), function (item) {
      return item !== 'id';
    });
    if (!rstItem.hasWroteHead) {
      /* add csv header */
      hdr = getRcdIdHdr();
      hdr = hdr.concat(datKeys);
      rstItem.csv.writeRecord(hdr);
      rstItem.hasWroteHead = true;
    }
    /* add csv line */
    csvLine = getRcdIdCsv(rcd.data.id);
    _.each(datKeys, function (item) {
      csvLine.push(rcd.data[item]);
    });
    rstItem.csv.writeRecord(csvLine);
  }

  function close() {
    dbgTrace('close all reporter');
    _.each(_.keys(rstFiles), function (item) {
      var rstItem;
      rstItem = rstFiles[item] || {};
      if (rstItem.fd) {
        rstItem.fd.end();
        rstItem.fd = undefined;
        rstItem.csv = undefined;
      }
    });
  }

  return { init: init, addRcd: addRcd, close: close };
};

