/*
 * #© Copyright 2019 - Micro Focus or one of its affiliates
 * #
 * # The only warranties for products and services of Micro Focus and its affiliates and licensors (“Micro Focus”)
 * # are as may be set forth in the express warranty statements accompanying such products and services.
 * # Nothing herein should be construed as constituting an additional warranty.
 * # Micro Focus shall not be liable for technical or editorial errors or omissions contained herein.
 * # The information contained herein is subject to change without notice.
 *
 */

'use strict';

var _ = require('underscore'),
  path = require('path'),
  fs = require('fs'),
  misc = require('./misc.js'),
  dbgTrace = misc.dbgTrace('testGen'),
  logger = require('./logger.js');

exports.create = function (opts) {
  var template, templateCfg;

  /* load template */
  opts = opts || {};
  opts.templatesDir = opts.templatesDir || path.join(__dirname, '../templates');
  opts.templatesDir = path.resolve(opts.templatesDir);
  opts.template = opts.template || 'default';
  dbgTrace('load template %j', opts);
  templateCfg = path.join(opts.templatesDir, opts.template, 'template.json');
  dbgTrace('load template cfg %s', templateCfg);
  template = misc.loadJSONSync(templateCfg);
  dbgTrace('loaded template %j', template);
  if (!template || !template.projectFiles || !template.testScript) {
    logger.error('Cannot load project template %s', templateCfg);
    return undefined;
  }

  function genProjectItem(testOpts, prjItem) {
    var src, srcFile, targetFile, targetTemplate, srcTemplate;
    dbgTrace('gen prjItem %j', prjItem);
    if (!prjItem.src || !prjItem.target) {
      /* empty project item */
      return;
    }

    /* target file */
    targetTemplate = _.template(prjItem.target);
    targetFile = targetTemplate(testOpts);
    targetFile = path.join(testOpts.dir, targetFile);
    dbgTrace('target file %s', targetFile);

    /* load src */
    srcFile = path.join(opts.templatesDir, opts.template, prjItem.src);
    src = fs.readFileSync(srcFile);
    src = src.toString();
    srcTemplate = _.template(src);
    src = srcTemplate(testOpts);
    
    /* write to target */
    fs.writeFileSync(targetFile, src);
  }

  /* generate project */
  function gen(testOpts, callback) {
    var retErr, testScript;
    testOpts = testOpts || {};
    callback = callback || function () {};
    dbgTrace('start create test %j. generator %j', testOpts, opts);
    if (!testOpts.dir || !testOpts.testName) {
      callback(new Error('Invalid test options'));
      return;
    }
    /* update test setting */
    testScript =  _.template(template.testScript)(testOpts);
    testOpts.testScript = testScript;

    /* generate project items */
    retErr = undefined;
    _.each(template.projectFiles, function (prjItem) {
      try {
        genProjectItem(testOpts, prjItem);
      }
      catch (err) {
        logger.error('Cannot generate item %s', err.toString());
        retErr = err;
      }
    });
    callback(retErr);
  }

  return { gen: gen };
};

