/*
 * #© Copyright 2019 - Micro Focus or one of its affiliates
 * #
 * # The only warranties for products and services of Micro Focus and its affiliates and licensors (“Micro Focus”)
 * # are as may be set forth in the express warranty statements accompanying such products and services.
 * # Nothing herein should be construed as constituting an additional warranty.
 * # Micro Focus shall not be liable for technical or editorial errors or omissions contained herein.
 * # The information contained herein is subject to change without notice.
 *
 */

'use strict';

var vuserContext = require('./vuser_context.js'),
  VUser = require('./vuser.js').VUser,
  logger = require('./logger.js');

exports = module.exports = function (opts) {
  var vuCache, runtimeCfg, meter, runner;

  /* vuser cache data
   * {
   *    'vuserId - 1': vuObj,
   *    'vuserId - 2': vuObj,
   *    ...
   * }
   */
  vuCache = {};

  /* startup vuser manager */
  opts = opts || {};
  runtimeCfg = opts.runtimeCfg;
  meter = opts.meter;
  runner = opts.runner;

  function getVUser(vuId) {
    if (!vuCache[vuId]) {
      logger.error('Bad Vuser Id %d', vuId);
      return undefined;
    }
    return vuCache[vuId];
  }

  function getAllVUsers() {
    return vuCache;
  }

  function createNewVUser(opts) {
    var vuCtx, vu;
    vuCtx = vuserContext.create(opts);
    vu = new VUser({
      vuCtx: vuCtx,
      meter: meter,
      cfg: runtimeCfg,
      runner: runner
    });
    vuCache[opts.vuserId] = vu;
    return vu;
  }

  return {
    createNewVUser: createNewVUser,
    getAllVUsers: getAllVUsers,
    getVUser: getVUser
  };
};

