/*
 * #© Copyright 2019 - Micro Focus or one of its affiliates
 * #
 * # The only warranties for products and services of Micro Focus and its affiliates and licensors (“Micro Focus”)
 * # are as may be set forth in the express warranty statements accompanying such products and services.
 * # Nothing herein should be construed as constituting an additional warranty.
 * # Micro Focus shall not be liable for technical or editorial errors or omissions contained herein.
 * # The information contained herein is subject to change without notice.
 *
 */

'use strict';

var vuSvcTran = require('./vuser_svc_transaction.js'),
  vuSvcMsg = require('./vuser_svc_message.js'),
  vuSvcDP = require('./vuser_svc_datapoint.js'),
  vuSvcHttp = require('./vuser_svc_http.js'),
  vuSvcThinkTime = require('./vuser_svc_thinktime.js'),
  vuSvcReqeust = require('./vuser_svc_request.js'),
  measurement = require('./measurement.js'),
  _ = require('underscore');

exports = module.exports = function (opts) {
  var svc, baseSvcOpts;

  baseSvcOpts = { vuCtx: opts.vuCtx, meter: opts.meter, cfg: opts.cfg };
  /* base service */
  svc = {
    _datapoint: vuSvcDP(baseSvcOpts),
    logger: vuSvcMsg(baseSvcOpts)
  };

  /* export datapoint service */
  svc.datapoint = svc._datapoint;

  /* measurement service */
  svc._measurement = {
    http200: measurement(_.extend({dpSvc: svc._datapoint}, baseSvcOpts), 'HTTP_200'),
    recv: measurement(_.extend({dpSvc: svc._datapoint}, baseSvcOpts), 'mic_recv')
  };
  /* thinkTime service */
  svc.thinkTime = vuSvcThinkTime(_.extend({
    loggerSvc: svc.logger
  }, baseSvcOpts));

  /* transaction service */
  svc.transaction = vuSvcTran(_.extend({
    thinkTimeSvc: svc.thinkTime
  }, baseSvcOpts));

  /* http service */
  svc.http = vuSvcHttp(_.extend({
    dpSvc: svc._datapoint,
    measurementSvc: svc._measurement,
    loggerSvc: svc.logger
  }, baseSvcOpts));

  /* request service */
  svc.request = vuSvcReqeust(_.extend({
    dpSvc: svc._datapoint,
    measurementSvc: svc._measurement,
    loggerSvc: svc.logger
  }, baseSvcOpts));

  return svc;
};

