/*
 * #© Copyright 2019 - Micro Focus or one of its affiliates
 * #
 * # The only warranties for products and services of Micro Focus and its affiliates and licensors (“Micro Focus”)
 * # are as may be set forth in the express warranty statements accompanying such products and services.
 * # Nothing herein should be construed as constituting an additional warranty.
 * # Micro Focus shall not be liable for technical or editorial errors or omissions contained herein.
 * # The information contained herein is subject to change without notice.
 *
 */

'use strict';

var http = require('http'),
  util = require('util'),
  _ = require('underscore'),
  misc = require('./misc.js'),
  dbgTrace = misc.dbgTrace('svcHttp');

exports = module.exports = function (opts) {
  var dpSvc, measurementSvc, loggerSvc, vuCtx;

  opts = opts || {};
  dpSvc = opts.dpSvc;
  measurementSvc = opts.measurementSvc;
  loggerSvc = opts.loggerSvc;
  vuCtx = opts.vuCtx;

  function getVuHttpAgent() {
    return false;
    // if (!vuHttpAgent) {
    //   dbgTrace('create new vuHttpAgent, vu = %s', vuCtx.getVUserId());
    //   vuHttpAgent = new http.Agent();
    // }
    // return vuHttpAgent;
  }

  function report(data) {
    var name;
    dbgTrace('new http dp, %j', data);
    if (data.statusCode === 200) {
      measurementSvc.http200.add(1);
    } else {
      name = util.format('HTTP_%d', data.statusCode);
      dpSvc.add(name, 1);
    }
  }

  function httpBase(method, options, cb) {
    var beginTS, reqObj, bodyLen, originalBytesRead, callback;

    dbgTrace('new http request: vu=%s, opt=%j', vuCtx.getVUserId(), options);
    options = options || {};
    if (_.isUndefined(options.agent)) {
      options.agent = getVuHttpAgent();
    }

    cb = cb || function () {};
    callback = function (res) {
      try {
        cb(res);
      }
      catch (ex) {
        loggerSvc.error(misc.messageIDs.testScriptErr,
          '%s', ex.stack.toString());
        setImmediate(function () { throw ex; });
      }
    };

    beginTS = Date.now();
    bodyLen = 0;
    originalBytesRead = 0;
    reqObj = http[method](options, function(res) {
      var endTS;

      res.on('data', function (chunk) {
        if (chunk && chunk.length > 0) {
          bodyLen += chunk.length;
        }
      }).on('end', function () {
        if (res && res.socket &&
            res.socket.bytesRead > 0 &&
            res.socket.bytesRead - originalBytesRead > 0) {
          measurementSvc.recv.add(res.socket.bytesRead - originalBytesRead);
        }
        else if (bodyLen > 0) {
          measurementSvc.recv.add(bodyLen);
        }
      }).on('socket', function (sock) {
        if (sock && sock.bytesRead > 0) {
          originalBytesRead = sock.bytesRead;
        }
      });

      endTS = Date.now();
      report({ beginTS: beginTS, endTS: endTS, statusCode: res.statusCode });
      callback(res);
    });

    return reqObj;
  }

  return {
    request: _.partial(httpBase, 'request'),
    get: _.partial(httpBase, 'get')
  };
};
 
