/*
 * #© Copyright 2019 - Micro Focus or one of its affiliates
 * #
 * # The only warranties for products and services of Micro Focus and its affiliates and licensors (“Micro Focus”)
 * # are as may be set forth in the express warranty statements accompanying such products and services.
 * # Nothing herein should be construed as constituting an additional warranty.
 * # Micro Focus shall not be liable for technical or editorial errors or omissions contained herein.
 * # The information contained herein is subject to change without notice.
 *
 */

'use strict';

var util = require('util'),
  _ = require('underscore'),
  misc = require('./misc.js'),
  dbgTrace = misc.dbgTrace('svcMsg'),
  logger = require('./logger.js'),
  msgTypeConverter;

msgTypeConverter = (function () {
  /* Error - 0, Warning - 1, Notification - 2 */
  var msgType = { error: 0, warn: 1, info: 2 };
  return {
    getMsgType: function (level) {
      level = level || 'info';
      return _.isUndefined(msgType[level]) ? msgType.info : msgType[level];
    }
  };
})();

exports = module.exports = function (opts) {
  var vuCtx, meter;
  opts = opts || {};
  vuCtx = opts.vuCtx;
  meter = opts.meter;

  function report(opts) {
    var fsLogger;

    opts = opts || {};
    dbgTrace('msg: %j', opts);
    fsLogger = logger[opts.level] || logger.error;
    fsLogger('vuser %s, %s[%d]: %s',
      vuCtx.getVUserId(), opts.level, opts.msgId, opts.msg);
    meter.message(vuCtx, {
      level: opts.level,
      msgId: opts.msgId,
      msg: opts.msg,
      msgType: msgTypeConverter.getMsgType(opts.level),
      ts: Date.now()
    });
  }

  function checkRepArguments() {
    return {
      msgId: _.isNumber(arguments[0]) ?
              Array.prototype.shift.apply(arguments) : 0,
      msg: util.format.apply(null, arguments)
    };
  }

  return {
    error: function () {
      var opts;
      opts = checkRepArguments.apply(null, arguments);
      opts.level = 'error';
      report(opts);
    },
    warn: function () {
      var opts;
      opts = checkRepArguments.apply(null, arguments);
      opts.level = 'warn';
      report(opts);
    },
    info: function () {
      var opts;
      opts = checkRepArguments.apply(null, arguments);
      opts.level = 'info';
      report(opts);
    }
  };
};

