/*
 * #© Copyright 2019 - Micro Focus or one of its affiliates
 * #
 * # The only warranties for products and services of Micro Focus and its affiliates and licensors (“Micro Focus”)
 * # are as may be set forth in the express warranty statements accompanying such products and services.
 * # Nothing herein should be construed as constituting an additional warranty.
 * # Micro Focus shall not be liable for technical or editorial errors or omissions contained herein.
 * # The information contained herein is subject to change without notice.
 *
 */

'use strict';

var requestLib = require('request'),
  url = require('url'),
  util = require('util'),
  misc = require('./misc.js'),
  logger = require('./logger.js'),
  _ = require('underscore'),
  dbgTrace = misc.dbgTrace('svcRequest');

exports = module.exports = function (opts) {
  var dpSvc, measurementSvc, loggerSvc, vuCtx, defaultTimeout;

  opts = opts || {};
  dpSvc = opts.dpSvc;
  measurementSvc = opts.measurementSvc;
  loggerSvc = opts.loggerSvc;
  vuCtx = opts.vuCtx;
  defaultTimeout = 1000 * 30;

  function getVuAgent(protocol) {
    return false;
    // if (protocol === 'https:') {
    //   vuHttpsAgent = vuHttpsAgent || new https.Agent();
    //   return vuHttpsAgent;
    // }
    // else {
    //   vuHttpAgent = vuHttpAgent || new http.Agent();
    //   return vuHttpAgent;
    // }
  }

  function report(data) {
    var name;
    dbgTrace('new dp, %j', data);
    if (data.statusCode === 200) {
      measurementSvc.http200.add(1);
    } else {
      name = util.format('HTTP_%d', data.statusCode);
      dpSvc.add(name, 1);
    }
  }

  function request(uri, options, callback) {
    var urlObj, beginTS, bkUri, bkOpts, bkCallback,
      reqObj, originalBytesRead, cb;

    bkUri = uri;
    bkOpts = options;
    bkCallback = callback;
    if (_.isUndefined(uri)) {
      logger.error('undefined uri object');
      return requestLib(uri, options, callback);
    }
    if ((typeof options === 'function') && !callback) {
      callback = options;
    }
    if (options && typeof options === 'object') {
      options.uri = uri;
    }
    else if (typeof uri === 'string') {
      options = {uri: uri};
    }
    else {
      options = uri;
      uri = options.uri;
    }
    try {
      urlObj = url.parse(uri || options.url);
    }
    catch (err) {
      logger.error('vuser request svc, cannot parse url');
      return requestLib(bkUri, bkOpts, bkCallback);
    }
    if (urlObj.protocol !== 'http:' && urlObj.protocol !== 'https:') {
      logger.error('vuser request svc, cannot parse url');
      return requestLib(bkUri, bkOpts, bkCallback);
    }

    if (_.isUndefined(options.agent)) {
      options.agent = getVuAgent(urlObj.protocol);
    }
    if (_.isUndefined(options.timeout)) {
      options.timeout = defaultTimeout;
    }

    dbgTrace('new request, vu = %s, opt %j', vuCtx.getVUserId(), options);
    beginTS = Date.now();
    originalBytesRead = 0;

    cb = callback || function () {};
    callback = function (err, res, body) {
      try {
        cb(err, res, body);
      }
      catch (ex) {
        loggerSvc.error(misc.messageIDs.testScriptErr,
          '%s', ex.stack.toString());
        setImmediate(function () { throw ex; });
      }
    };

    reqObj = requestLib(options, function (err, res, body) {
      var endTS, data;

      if (err || !res || _.isUndefined(res.statusCode)) {
        callback(err, res, body);
        return;
      }

      if (res && res.socket &&
          res.socket.bytesRead > 0 &&
          res.socket.bytesRead - originalBytesRead > 0) {
        measurementSvc.recv.add(res.socket.bytesRead - originalBytesRead);
      }
      else if (body && body.length > 0) {
        measurementSvc.recv.add(body.length);
      }

      endTS = Date.now();
      data = { beginTS: beginTS, endTS: endTS, statusCode: res.statusCode };
      report(data);
      callback(err, res, body);
    });

    reqObj.on('request', function(realReq) {
      realReq.on('socket', function (sock) {
        if (sock && sock.bytesRead > 0) {
          originalBytesRead = sock.bytesRead;
        }
      });
    });
    return reqObj;
  }

  return request;
};

