/*
 * #© Copyright 2019 - Micro Focus or one of its affiliates
 * #
 * # The only warranties for products and services of Micro Focus and its affiliates and licensors (“Micro Focus”)
 * # are as may be set forth in the express warranty statements accompanying such products and services.
 * # Nothing herein should be construed as constituting an additional warranty.
 * # Micro Focus shall not be liable for technical or editorial errors or omissions contained herein.
 * # The information contained herein is subject to change without notice.
 *
 */

'use strict';

var misc = require('./misc.js'),
  dbgTrace = misc.dbgTrace('svcThinkTM'),
  _ = require('underscore');

exports = module.exports = function (opts) {
  var loggerSvc;

  /* startup thinkTime service */
  opts = opts || {};
  loggerSvc = opts.loggerSvc;

  function thinkTime(delay, cb) {
    var callback;

    cb = cb || function () {};
    callback = function (err) {
      try {
        cb(err);
      }
      catch (ex) {
        loggerSvc.error(misc.messageIDs.testScriptErr,
          '%s', ex.stack.toString());
        setImmediate(function () { throw ex; });
      }
    };

    if (_.isNaN(delay) || !_.isNumber(delay)) {
      loggerSvc.error('Cannot call thinkTime: Invalid arguments');
      callback(new Error('Invalid arguments'));
      return;
    }
    dbgTrace('thinkTM (%d), delay', delay);
    setTimeout(callback, delay);
  }
  return thinkTime;
};

