/*
 * #© Copyright 2019 - Micro Focus or one of its affiliates
 * #
 * # The only warranties for products and services of Micro Focus and its affiliates and licensors (“Micro Focus”)
 * # are as may be set forth in the express warranty statements accompanying such products and services.
 * # Nothing herein should be construed as constituting an additional warranty.
 * # Micro Focus shall not be liable for technical or editorial errors or omissions contained herein.
 * # The information contained herein is subject to change without notice.
 *
 */

'use strict';

var logger = require('./logger.js'),
  _ = require('underscore'),
  misc = require('./misc.js'),
  dbgTrace = misc.dbgTrace('svcTran');

exports = module.exports = function (opts) {
  var vuCtx, meter, statusTypes, allTran, thinkTMSvc, inst;

  /* check args */
  opts = opts || {};
  vuCtx = opts.vuCtx;
  meter = opts.meter;
  statusTypes = { PASS: 'pass', FAIL: 'fail', STOP: 'stop' };
  thinkTMSvc = opts.thinkTimeSvc;

  /* all transactions 
   * {
   *  'name1': { beginTS: ts, endTS: ts, tranStatus: 'pass', thinkTM: 0 }
   *  'name2': { beginTS: ts, endTS: ts, tranStatus: 'pass', thinkTM: 0 }
   *  ...
   * }
   */
  allTran = {};

  function getStatusCode(statusType) {
    if (statusType === statusTypes.FAIL || statusType === statusTypes.STOP) {
      return 1;
    }
    else {
      return 0;
    }
  }

  function report(name, tran) {
    meter.transaction(vuCtx, {
      name: name,
      beginTS: tran.beginTS / 1000,
      endTS: tran.endTS / 1000,
      tranStatus: tran.tranStatus,
      thinkTM: tran.thinkTM / 1000,
      ts: Date.now(),
      statusCode: getStatusCode(tran.tranStatus)
    });
  }

  function start(name) {
    var now;
    misc.tranNames.addTran(name);
    name = name || '';
    now = Date.now();
    dbgTrace('start tran %s', name);
    if (allTran[name]) {
      logger.warn('transaction name (%s) already exists', name);
    }
    allTran[name] = {
      beginTS: now,
      endTS: now,
      tranStatus: statusTypes.PASS,
      thinkTM: 0
    };
  }

  function end(name, tranStatus) {
    var tran;
    name = name || '';
    tranStatus = tranStatus || statusTypes.PASS;
    dbgTrace('end tran %s, status %s', name, tranStatus);
    tran = allTran[name];
    if (!tran) {
      logger.error('Cannot find tran %s, vu %s', name, vuCtx.getVUserId());
      return;
    }
    tran.endTS = Date.now();
    tran.tranStatus = tranStatus;
    report(name, tran);
    delete allTran[name];
  }

  function thinkTime(name, tm, callback) {
    var tran;
    name = name || '';
    tm = tm || 0;
    dbgTrace('think time %s, tm %d', name, tm);
    tran = allTran[name];
    if (!tran) {
      logger.error('Cannot find tran %s, vu %s', name, vuCtx.getVUserId());
    }
    else {
      tran.thinkTM = tran.thinkTM + tm;
    }
    if (callback) {
      dbgTrace('set think time callback %d', tm);
      thinkTMSvc(tm, callback);
    }
  }

  inst = _.extend({start: start, end: end, thinkTime: thinkTime}, statusTypes);
  return inst;
};

