/*
 * #© Copyright 2019 - Micro Focus or one of its affiliates
 * #
 * # The only warranties for products and services of Micro Focus and its affiliates and licensors (“Micro Focus”)
 * # are as may be set forth in the express warranty statements accompanying such products and services.
 * # Nothing herein should be construed as constituting an additional warranty.
 * # Micro Focus shall not be liable for technical or editorial errors or omissions contained herein.
 * # The information contained herein is subject to change without notice.
 *
 */

'use strict';

var tunnel = require('tunnel-agent'),
  url = require('url');

exports = module.exports = function (vuser) {
  var requestOptions;

  vuser.init('simple HTTP request tests', function(svc, done) {
    var proxy;
    svc.logger.info('Init http test %s', vuser.getVUserId());
    requestOptions = { hostname: 'nodejs.org', port: 80, path: '/' };
    if (process.env.http_proxy) {
      /* update the http_proxy setting */
      proxy = url.parse(process.env.http_proxy);
      svc.logger.info('set http proxy %s:%s', proxy.hostname, proxy.port);
      requestOptions.agent = tunnel.httpOverHttp({
        maxSockets: 100,
        proxy: { host: proxy.hostname, port: proxy.port }
      });
    }
    done();
  });

  vuser.action('simple HTTP request tests', function (svc, done) {
    svc.logger.info('Start http test %s', vuser.getVUserId());

    /* send request to server */
    svc.transaction.start('httpTest');
    function sendRequest() {
      svc.http.request(requestOptions, function (res) {
        svc.logger.info('http response statusCode = %d', res.statusCode);
        res.on('end', function() {
          svc.transaction.end('httpTest', svc.transaction.PASS);
          done();
        }).on('error', function() {
          svc.transaction.end('httpTest', svc.transaction.FAIL);
          done();
        });
      }).on('error', function (err) {
        svc.logger.error('http error %s', err.toString());
        svc.transaction.end('httpTest', svc.transaction.FAIL);
        done();
      }).end();
    }

    svc.transaction.thinkTime('httpTest', 1000 * 2, function () {
      sendRequest();
    });
  });
};

