/*
 * #© Copyright 2019 - Micro Focus or one of its affiliates
 * #
 * # The only warranties for products and services of Micro Focus and its affiliates and licensors (“Micro Focus”)
 * # are as may be set forth in the express warranty statements accompanying such products and services.
 * # Nothing herein should be construed as constituting an additional warranty.
 * # Micro Focus shall not be liable for technical or editorial errors or omissions contained herein.
 * # The information contained herein is subject to change without notice.
 *
 */

/* Test name:    urls_test
 * Script name:  urls_test.js
 */
'use strict';

var path = require('path');

exports = module.exports = function (vuser) {
  var vuserId, proxy, urlListFile, urlList;
 
  /* prepare test data */
  vuserId = vuser.getVUserId();
  urlListFile = 'urls.json';
  urlList = [];
  proxy = process.env.http_proxy ? process.env.http_proxy : undefined;

  /* init action */
  vuser.init('Vuser init action', function (svc, done) {
    svc.logger.info('Vuser %s init', vuserId);

    /* load url list */
    urlListFile = path.resolve(__dirname, urlListFile);
    svc.logger.info('load url list from %s', urlListFile);
    try {
      urlList = require(urlListFile);
    }
    catch (err) {
      svc.logger.error('Cannot load url list from %s', urlListFile);
    }
    done();
  });

  /* test URL item */
  function testUrlItem(svc, urlItem, callback) {
    var reqOpts;

    callback = callback || function () {};
    urlItem = urlItem || {};
    reqOpts = {
      url: urlItem.url,
      proxy: proxy
    };
    svc.logger.info('Testing URL %s', urlItem.url);
    svc.request(reqOpts, function (err, res, body) {
      if (err) {
        svc.logger.error('request error %s', err.toString());
      }
      callback();
    });
  }

  /* main action */
  vuser.action('Vuser main action', function (svc, done) {
    var idx, len;

    idx = 0;
    len = urlList.length;
    svc.logger.info('Test Url list length is %d', len);
    if (urlList.length <= 0) {
      svc.logger.error('An invalid Url list.');
      done();
      return;
    }

    function onCallback(err) {
      if (err) {
        svc.logger.error('Error: %s', err.toString());
      }
      idx++;
      if (idx < len) {
        /* test the next url */
        svc.thinkTime(1000 * 3, function () {
          testUrlItem(svc, urlList[idx], onCallback);
        });
      }
      else {
        done();
      }
    }
    /* test the first url */
    svc.thinkTime(1000 * 3, function () {
      testUrlItem(svc, urlList[idx], onCallback);
    });
  });
};

